/**
  Draw multiple objects in a single raytracer pass.
  
  Dr. Orion Sky Lawlor, olawlor@acm.org, 2012-01-30 (Public Domain)
*/
#include "physics/world.h" /* physics::library and physics::object */
#include "ogl/glsl.h"
#include "soil/SOIL.h"

/* Just include library bodies here, for easy linking */
#include "physics/world.cpp" 
#include "physics/config.cpp"
#include "ogl/glsl.cpp"

#include "soil/SOIL.c"
#include "soil/stb_image_aug.c"



/* A raytraced object */
class rayObject : public physics::object {
public:
	rayObject(void) 
		:physics::object(0.01)  /* <- our timestep, in seconds */
	{ }
	
	void simulate(physics::library &lib) { }
	
	void draw(physics::library &lib) {
		static GLuint texHandle=SOIL_load_OGL_texture(
			"cloud.jpg",
			4, /* RGBA */
			0, /* new texture ID */
			SOIL_FLAG_MIPMAPS | SOIL_FLAG_TEXTURE_REPEATS | SOIL_FLAG_INVERT_Y
		);
	
		/* Load up shader */
		static programFromFiles prog; 
		prog.load("vertex.txt","fragment.txt");
		glUseProgramObjectARB(prog);
		unsigned int ul=glGetUniformLocationARB(prog,"camera");
		glUniform3fvARB(ul,1,camera);
		
		ul=glGetUniformLocationARB(prog,"time");
		glUniform1fARB(ul,0.001*glutGet(GLUT_ELAPSED_TIME));
		
		glActiveTexture(GL_TEXTURE0); /* select texture unit 0 */
		glBindTexture(GL_TEXTURE_2D,texHandle); /* bind handle */
		ul=glGetUniformLocationARB(prog,"texSampler");
		glUniform1iARB(ul,0); /* set sampler to texture unit 0 */

		/* Draw raytracer proxy geometry *HUGE* (to cover everything) */
		glColor4f(1.0f,1.0f,1.0f,0.4f);
		glutSolidSphere(500.0,4,8);
		
		glUseProgramObjectARB(0);
	}
};


/* Called to create a new simulation */
void physics_setup(physics::library &lib) {
	lib.world->add(new rayObject());
	lib.background[0]=lib.background[1]=0.6;
	lib.background[2]=1.0; // light blue
}

/* Called every frame to draw everything */
void physics_draw_frame(physics::library &lib) {
	/* Point and line smoothing needs usual alpha blending */
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	
	lib.world->draw(lib);
}


